'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _errors = require('../errors');

var _config = require('../config');

var _driverLogging = require('../driver-logging');

var _configProvider = require('../config-provider');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var loadingLabelMap = {
  destroy: 'Destroying services...',
  start: 'Starting services...',
  stop: 'Stopping services...',
  restart: 'Restarting services...',
  kill: 'Killing services...',
  rm: 'Removing services...',
  update: 'Updating service images...',
  delete: 'Deleting navy configuration...',
  useTag: 'Pulling custom tag...',
  resetTag: 'Resetting to default tag...',
  usePort: 'Setting up port mapping...',
  resetPort: 'Resetting port mapping...'
};

function removeFirstLineFromStackTrace(ex) {
  return ex.stack.split('\n').slice(1).join('\n');
}

function wrapper(res) {
  if (res.catch) {
    res.catch(function (ex) {
      (0, _driverLogging.stopDriverLogging)({ success: false });

      if (ex instanceof _errors.NavyError) {
        ex.prettyPrint();
      } else if (ex.name === 'Invariant Violation') {
        console.log();
        console.log(_chalk2.default.bgRed(_chalk2.default.bold(' ' + ex.name + ' ')));
        console.log();
        console.log(' ' + ex.message.substring(ex.message.indexOf(': ') + 2));
        console.log();
        console.log(' ' + _chalk2.default.blue('Run') + ' ' + _chalk2.default.bold('navy doctor') + ' ' + _chalk2.default.blue('to attempt troubleshooting'));
        console.log();
        console.log(_chalk2.default.dim(removeFirstLineFromStackTrace(ex.stack)));
        console.log();
      } else {
        console.error(ex.stack);
      }

      process.exit(1);
    });
  }

  return res;
}

function basicCliWrapper(fnName) {
  var wrapperOpts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var driverLogging = wrapperOpts.driverLogging == null ? true : wrapperOpts.driverLogging;

  return function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(maybeServices) {
      for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      var _require, getNavy, opts, otherArgs, envName, navy, returnVal;

      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _require = require('../navy'), getNavy = _require.getNavy;
              opts = args.length === 0 ? maybeServices : args[args.length - 1];
              otherArgs = args.slice(0, args.length - 1);
              envName = opts.navy;


              if (wrapperOpts.serviceBasedAlias && maybeServices.length) {
                console.log('This command should not be called with a list of services. calling \'' + wrapperOpts.serviceBasedAlias + '\' instead');
                fnName = wrapperOpts.serviceBasedAlias;
                maybeServices = maybeServices.split(' ');
              }

              process.on('unhandledRejection', function (ex) {
                (0, _driverLogging.stopDriverLogging)({ success: false });

                if (ex instanceof _errors.NavyError) {
                  ex.prettyPrint();
                } else {
                  console.error(ex.stack);
                }

                process.exit();
              });

              if (driverLogging) (0, _driverLogging.startDriverLogging)(loadingLabelMap[fnName]);

              navy = getNavy(envName);
              _context.next = 10;
              return navy.ensurePluginsLoaded();

            case 10:
              _context.next = 12;
              return navy.emitAsync('cli.before.' + fnName, fnName);

            case 12:
              _context.next = 14;
              return navy[fnName].apply(navy, [Array.isArray(maybeServices) && maybeServices.length === 0 ? undefined : maybeServices].concat((0, _toConsumableArray3.default)(otherArgs)));

            case 14:
              returnVal = _context.sent;
              _context.next = 17;
              return navy.emitAsync('cli.after.' + fnName, fnName);

            case 17:

              if (driverLogging) (0, _driverLogging.stopDriverLogging)();

              if (!Array.isArray(returnVal)) {
                _context.next = 20;
                break;
              }

              return _context.abrupt('return', console.log(returnVal.join('\n')));

            case 20:

              if (returnVal != null) {
                console.log(returnVal);
              }

            case 21:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function (_x2) {
      return _ref.apply(this, arguments);
    };
  }();
}

function lazyRequire(path) {
  return function () {
    var mod = require(path);
    return wrapper((mod.default || mod).apply(undefined, arguments));
  };
}

var defaultNavy = process.env.NAVY_NAME || (0, _config.getConfig)().defaultNavy;

var importCommand = _commander2.default.command('import').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Imports docker compose configuration from the current working directory and initialises a new navy').action(lazyRequire('./import'));

(0, _configProvider.getImportCommandLineOptions)().forEach(function (opt) {
  return importCommand.option.apply(importCommand, (0, _toConsumableArray3.default)(opt));
});

_commander2.default.command('launch [services...]').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Launches the given services in a navy').action(lazyRequire('./launch')).on('--help', function () {
  return console.log('\n  This will prompt you for the services that you want to bring up.\n  You can optionally provide the names of services to bring up which will disable the interactive prompt.\n  ');
});

_commander2.default.command('destroy').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Destroys a navy and all related data and services').action(basicCliWrapper('destroy', { serviceBasedAlias: 'kill' })).on('--help', function () {
  return console.log('\n  This will destroy an entire navy and all of its data and services.\n\n  Examples:\n    $ navy destroy # destroy "' + defaultNavy + '" navy\n    $ navy destroy -e dev # destroy "dev" navy\n    $ navy destroy -e test # destroy "test" navy\n  ');
});

_commander2.default.command('delete').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Removes a navy configuration without removing any docker containers or services').action(basicCliWrapper('delete'));

_commander2.default.command('ps').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).option('--json', 'output JSON instead of a table').description('Lists the running services for a navy').action(lazyRequire('./ps'));

_commander2.default.command('start [services...]').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Starts the given services').action(basicCliWrapper('start'));

_commander2.default.command('stop [services...]').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Stops the given services').action(basicCliWrapper('stop'));

_commander2.default.command('restart [services...]').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Restarts the given services').action(basicCliWrapper('restart'));

_commander2.default.command('kill [services...]').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Kills the given services').action(basicCliWrapper('kill'));

_commander2.default.command('rm [services...]').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Removes the given services').action(basicCliWrapper('rm'));

_commander2.default.command('update [services...]').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Pulls the given services\' images from their respective registries and relaunches the services').action(basicCliWrapper('update'));

_commander2.default.command('updates').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Checks for updates for all the launched services').action(lazyRequire('./updates'));

_commander2.default.command('logs [services...]').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Streams logs for the given services').action(lazyRequire('./logs'));

_commander2.default.command('health').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Shows the health status for all of the launched services').action(lazyRequire('./health'));

_commander2.default.command('wait-for-healthy [services...]').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Waits for the given services to be healthy').action(lazyRequire('./wait-for-healthy'));

_commander2.default.command('use-tag <service> <tag>').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Uses a specific tag for the given service').action(basicCliWrapper('useTag'));

_commander2.default.command('reset-tag <service>').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Resets any tag override on the given service').action(basicCliWrapper('resetTag'));

_commander2.default.command('use-port <service> <internal> <external>').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Uses a specific external port for the given service and internal port').action(basicCliWrapper('usePort'));

_commander2.default.command('reset-port <service> <internal>').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Resets a specific external port mapping set by use-port').action(basicCliWrapper('resetPort'));

_commander2.default.command('url <service>').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Prints the external URL for the given service if it is a web service').action(basicCliWrapper('url', { driverLogging: false })).on('--help', function () {
  return console.log('\n  Examples:\n    $ navy url mywebserver\n    http://mywebserver.dev.127.0.0.1.nip.io\n  ');
});

_commander2.default.command('open <service>').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Opens the given service in the default web browser, if the service is configured with a URL').action(lazyRequire('./open')).on('--help', function () {
  return console.log('\n  Examples:\n    $ navy open mywebserver\n    Opening mywebserver...\n  ');
});

_commander2.default.command('port <service> <port>').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Prints the external port for the given internal port of the given service').action(basicCliWrapper('port', { driverLogging: false })).on('--help', function () {
  return console.log('\n  Examples:\n    $ navy port mywebserver 80\n    35821\n  ');
});

_commander2.default.command('available-services').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Prints the names of the services that are launched or can be launched').action(basicCliWrapper('getAvailableServiceNames', { driverLogging: false }));

_commander2.default.command('develop [service]').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Puts the given service into development using the current working directory').action(lazyRequire('./develop'));

_commander2.default.command('live [service]').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Takes the given service out of development').action(lazyRequire('./live'));

_commander2.default.command('run <name> [args...]').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Runs a named command specific to the given Navy').action(lazyRequire('./run'));

_commander2.default.command('refresh-config').option('-e, --navy [env]', 'set the navy name to be used [' + defaultNavy + ']', defaultNavy).description('Refreshes the configuration for the given Navy').action(lazyRequire('./refresh-config'));

_commander2.default.command('status').option('--json', 'output JSON instead of a table').description('List all of the imported navies').action(lazyRequire('./status'));

_commander2.default.command('doctor').description('Identifies and tries to fix some common issues which might cause Navy to stop working').action(lazyRequire('./doctor'));

_commander2.default.command('config').description('Modify and get Navy global config - run `navy config` for help').action(lazyRequire('./config/wrapper'));

_commander2.default.command('external-ip').description('Prints the external IP which services are accessible on').action(lazyRequire('./external-ip'));

_commander2.default.command('use-lan-ip').description('Use your LAN ip address for connecting to Navy services').action(lazyRequire('./lan-ip'));

_commander2.default.command('use-local-ip').description('Use your local ip address (127.0.0.1) for connecting to Navy services').action(lazyRequire('./local-ip'));

exports.default = _commander2.default;
module.exports = exports.default;