'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _ = require('../');

var _serviceHost = require('../util/service-host');

var _table = require('../util/table');

var _table2 = _interopRequireDefault(_table);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var SMALL_WINDOW_COLUMNS = 185;

function getStatus(service, state) {
  var statusString = service.status === 'exited' ? _chalk2.default.red(service.status) : service.status;

  if (state && state.services && state.services[service.name]) {
    var serviceState = state.services[service.name];

    if (serviceState._develop) {
      statusString += ' ' + _chalk2.default.yellow('(development)');
    }

    if (serviceState._tag) {
      statusString += ' ' + _chalk2.default.cyan('@ ' + serviceState._tag);
    }
  }

  return statusString;
}

function getPorts(service) {
  if (!service || !service.raw || !service.raw.NetworkSettings || !service.raw.NetworkSettings.Ports) {
    return '-';
  }

  var ports = service.raw.NetworkSettings.Ports;
  var portKeys = (0, _keys2.default)(ports);

  return portKeys.map(function (portKey) {
    return ports[portKey] ? ports[portKey].map(function (conf) {
      return conf.HostPort;
    }).join(', ') + '->' + portKey : portKey;
  }).join(', ');
}

var getDisplayUrl = function getDisplayUrl(serviceUrl) {
  return serviceUrl != null ? serviceUrl : '-';
};

exports.default = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(opts) {
    var navy, ps, state, isSmallConsole;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            navy = (0, _.getNavy)(opts.navy);
            _context.next = 3;
            return navy.ps();

          case 3:
            ps = _context.sent;
            _context.next = 6;
            return navy.getState();

          case 6:
            state = _context.sent;


            // $FlowIgnore getWindowSize not on type
            isSmallConsole = process.stdout.isTTY && process.stdout.getWindowSize()[0] < SMALL_WINDOW_COLUMNS;

            if (!opts.json) {
              _context.next = 10;
              break;
            }

            return _context.abrupt('return', console.log((0, _stringify2.default)(ps, null, 2)));

          case 10:
            if (!isSmallConsole) {
              _context.next = 12;
              break;
            }

            return _context.abrupt('return', console.log((0, _table2.default)([['ID', 'NAME', 'STATUS', 'PORTS', 'URL']].concat((0, _toConsumableArray3.default)(ps.map(function (service) {
              return [service.id.substring(0, 12), service.name, getStatus(service, state), getPorts(service), getDisplayUrl((0, _serviceHost.getUrlFromService)(service))];
            }))))));

          case 12:

            console.log((0, _table2.default)([['ID', 'NAME', 'IMAGE', 'STATUS', 'PORTS', 'URL']].concat((0, _toConsumableArray3.default)(ps.map(function (service) {
              return [service.id.substring(0, 12), service.name, service.image, getStatus(service, state), getPorts(service), getDisplayUrl((0, _serviceHost.getUrlFromService)(service))];
            })))));

          case 13:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function (_x) {
    return _ref.apply(this, arguments);
  };
}();

module.exports = exports.default;