'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.startDriverLogging = startDriverLogging;
exports.stopDriverLogging = stopDriverLogging;
exports.isDriverLogging = isDriverLogging;
exports.log = log;

var _cliSpinners = require('cli-spinners');

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _isDriverLogging = false;
var _spinnerInterval = null;
var _spinnerIndex = 0;
var _message = null;

function _redraw() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var symbol = _chalk2.default.cyan(_cliSpinners.dots.frames[_spinnerIndex]);

  if (opts.success === true) {
    symbol = _chalk2.default.green('✔');
  } else if (opts.success === false) {
    symbol = _chalk2.default.red('•');
  }

  if (!process.stdout.isTTY) {
    if (opts.success) {
      console.log();
      console.log(symbol, 'SUCCESS');
    } else if (opts.success === false) {
      console.log();
      console.log(symbol, 'FAILURE');
    }
    return;
  }

  process.stdout.write('                                                                        \n');
  process.stdout.write(' ' + symbol + ' ' + (opts.success === false ? _chalk2.default.red(_message, 'FAILED') : _message) + '\n');
  process.stdout.cursorTo(0);
  process.stdout.moveCursor(0, -2);
}

function startDriverLogging(message) {
  _isDriverLogging = true;
  _message = message;

  _redraw();

  _spinnerInterval = setInterval(function () {
    _spinnerIndex++;

    if (_spinnerIndex >= _cliSpinners.dots.frames.length) {
      _spinnerIndex = 0;
    }

    _redraw();
  }, _cliSpinners.dots.interval);
}

function stopDriverLogging() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  if (!isDriverLogging()) return;

  _isDriverLogging = false;
  clearInterval(_spinnerInterval);

  _redraw({ success: opts.success != null ? opts.success : true });

  if (process.stdout.isTTY) console.log('\n\n');
}

function isDriverLogging() {
  return _isDriverLogging;
}

function log(message) {
  if (!isDriverLogging()) return;

  process.stdout.write(_chalk2.default.dim(message));
}