'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

exports.createComposeClient = createComposeClient;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _fs = require('../../util/fs');

var _fs2 = _interopRequireDefault(_fs);

var _execAsync = require('../../util/exec-async');

var _driverLogging = require('../../driver-logging');

var _state = require('../../navy/state');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createComposeClient(navy) {
  var client = {
    exec: function exec(command) {
      var _this = this;

      var args = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
      var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        var composeArgs, useOriginalDockerComposeFiles, noLog, pipeLog, maxBuffer, composeOpts, composeFilePath;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                composeArgs = ['-p', navy.normalisedName];
                useOriginalDockerComposeFiles = opts.useOriginalDockerComposeFiles, noLog = opts.noLog, pipeLog = opts.pipeLog, maxBuffer = opts.maxBuffer;
                composeOpts = {
                  maxBuffer: maxBuffer || Infinity
                };
                _context.next = 5;
                return client.getDockerComposeFilePath();

              case 5:
                composeFilePath = _context.sent;

                if (!(composeFilePath && !useOriginalDockerComposeFiles)) {
                  _context.next = 10;
                  break;
                }

                composeArgs.push('-f', composeFilePath);
                _context.next = 13;
                break;

              case 10:
                _context.next = 12;
                return client.getOriginalDockerComposeDirectory();

              case 12:
                composeOpts.cwd = _context.sent;

              case 13:

                composeArgs.push.apply(composeArgs, [command].concat((0, _toConsumableArray3.default)(args)));

                _context.prev = 14;
                _context.next = 17;
                return (0, _execAsync.execAsync)('docker-compose', composeArgs, function (childProcess) {
                  if (!noLog && !pipeLog) {
                    childProcess.stdout.on('data', function (data) {
                      return (0, _driverLogging.log)(data);
                    });
                    childProcess.stderr.on('data', function (data) {
                      return (0, _driverLogging.log)(data);
                    });
                  } else if (pipeLog) {
                    childProcess.stdout.on('data', function (data) {
                      return process.stdout.write(data);
                    });
                    childProcess.stderr.on('data', function (data) {
                      return process.stderr.write(data);
                    });
                  }
                }, composeOpts);

              case 17:
                return _context.abrupt('return', _context.sent);

              case 20:
                _context.prev = 20;
                _context.t0 = _context['catch'](14);

                if (_context.t0.message && _context.t0.message.indexOf('Can\'t find a suitable configuration file') !== -1) {
                  (0, _invariant2.default)(false, 'NO_DOCKER_COMPOSE_FILE: No docker-compose.yml (or valid Docker Compose config) found');
                }

                throw _context.t0;

              case 24:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this, [[14, 20]]);
      }))();
    },
    getCompiledDockerComposePath: function getCompiledDockerComposePath() {
      return _path2.default.join((0, _state.pathToNavy)(navy.normalisedName), 'docker-compose.tmp.yml');
    },
    getOriginalDockerComposeDirectory: function getOriginalDockerComposeDirectory() {
      var _this2 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
        var configProvider;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return navy.getConfigProvider();

              case 2:
                configProvider = _context2.sent;


                (0, _invariant2.default)(configProvider, 'NO_CONFIG_PROVIDER: Couldn\'t determine config provider for navy %s', navy.name);

                return _context2.abrupt('return', configProvider.getNavyPath());

              case 5:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, _this2);
      }))();
    },
    getDockerComposeFilePath: function getDockerComposeFilePath() {
      var _this3 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
        var compiledComposePath;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.prev = 0;
                _context3.next = 3;
                return client.getCompiledDockerComposePath();

              case 3:
                compiledComposePath = _context3.sent;
                _context3.next = 6;
                return _fs2.default.statAsync(compiledComposePath);

              case 6:
                return _context3.abrupt('return', compiledComposePath);

              case 9:
                _context3.prev = 9;
                _context3.t0 = _context3['catch'](0);
                return _context3.abrupt('return', null);

              case 12:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, _this3, [[0, 9]]);
      }))();
    }
  };

  return client;
}