'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

exports.default = createDockerComposeDriver;

var _jsYaml = require('js-yaml');

var _jsYaml2 = _interopRequireDefault(_jsYaml);

var _fs = require('../../util/fs');

var _fs2 = _interopRequireDefault(_fs);

var _dockerClient = require('../../util/docker-client');

var _dockerClient2 = _interopRequireDefault(_dockerClient);

var _client = require('./client');

var _service = require('../../service');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = require('debug')('navy:docker-compose');

function getArgsFromOptions(opts, argMap) {
  return (0, _keys2.default)(opts).map(function (key) {
    return argMap[key];
  }).filter(function (arg) {
    return arg;
  });
}

var launchArgMap = {
  noDeps: '--no-deps',
  forceRecreate: '--force-recreate'
};

function createDockerComposeDriver(navy) {
  var _createComposeClient = (0, _client.createComposeClient)(navy),
      exec = _createComposeClient.exec,
      getCompiledDockerComposePath = _createComposeClient.getCompiledDockerComposePath;

  var driver = {
    launch: function launch(services) {
      var _this = this;

      var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        var additionalArgs;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                additionalArgs = [];


                debug('Got launch', services, opts);

                if (opts) {
                  additionalArgs.push.apply(additionalArgs, (0, _toConsumableArray3.default)(getArgsFromOptions(opts, launchArgMap)));
                  debug('Resolve opts to args', additionalArgs);
                }

                if (!services) {
                  services = [];
                }

                _context.next = 6;
                return exec('up', ['-d'].concat(additionalArgs, (0, _toConsumableArray3.default)(services)));

              case 6:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this);
      }))();
    },
    destroy: function destroy() {
      var _this2 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return exec('kill');

              case 2:
                _context2.next = 4;
                return exec('down', ['-v']);

              case 4:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, _this2);
      }))();
    },
    ps: function ps(service) {
      var _this3 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
        var projectName, psOptions, ps, inspect;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                projectName = navy.normalisedName;
                psOptions = {
                  all: true,
                  filters: {
                    label: ['com.docker.compose.project=' + projectName]
                  }
                };

                if (service) {
                  psOptions.filters.label.push('com.docker.compose.service=' + service);
                }

                _context3.next = 5;
                return _dockerClient2.default.listContainers(psOptions);

              case 5:
                ps = _context3.sent;
                _context3.next = 8;
                return _promise2.default.all(ps.map(function (container) {
                  return _dockerClient2.default.getContainer(container.Id).inspect();
                }));

              case 8:
                inspect = _context3.sent;
                return _context3.abrupt('return', inspect.map(function (service) {
                  return {
                    id: service.Id,
                    name: service.Config.Labels['com.docker.compose.service'],
                    image: service.Config.Image,
                    status: service.State.Running === true ? _service.Status.RUNNING : _service.Status.EXITED,
                    raw: service
                  };
                }));

              case 10:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, _this3);
      }))();
    },
    start: function start(services) {
      var _this4 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return exec('start', services);

              case 2:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, _this4);
      }))();
    },
    stop: function stop(services) {
      var _this5 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5() {
        return _regenerator2.default.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.next = 2;
                return exec('stop', services);

              case 2:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, _this5);
      }))();
    },
    restart: function restart(services) {
      var _this6 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6() {
        return _regenerator2.default.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _context6.next = 2;
                return exec('restart', services);

              case 2:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, _this6);
      }))();
    },
    kill: function kill(services) {
      var _this7 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7() {
        return _regenerator2.default.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                _context7.next = 2;
                return exec('kill', services);

              case 2:
              case 'end':
                return _context7.stop();
            }
          }
        }, _callee7, _this7);
      }))();
    },
    rm: function rm(services) {
      var _this8 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8() {
        return _regenerator2.default.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                if (!services) {
                  services = [];
                }

                _context8.next = 3;
                return exec('rm', ['-f', '-v'].concat((0, _toConsumableArray3.default)(services)));

              case 3:
              case 'end':
                return _context8.stop();
            }
          }
        }, _callee8, _this8);
      }))();
    },
    update: function update(services) {
      var _this9 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9() {
        var launchedServiceNames, servicesToRelaunch;
        return _regenerator2.default.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                if (!services) {
                  services = [];
                }

                _context9.next = 3;
                return exec('pull', services);

              case 3:
                _context9.t0 = toLookupTable;
                _context9.next = 6;
                return _this9.getLaunchedServiceNames();

              case 6:
                _context9.t1 = _context9.sent;
                launchedServiceNames = (0, _context9.t0)(_context9.t1);
                servicesToRelaunch = services.filter(function (name) {
                  return launchedServiceNames[name];
                });

                if (!servicesToRelaunch.length) {
                  _context9.next = 12;
                  break;
                }

                _context9.next = 12;
                return exec('up', ['-d', '--no-deps'].concat((0, _toConsumableArray3.default)(servicesToRelaunch)));

              case 12:
              case 'end':
                return _context9.stop();
            }
          }
        }, _callee9, _this9);
      }))();
    },
    spawnLogStream: function spawnLogStream(services) {
      var _this10 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10() {
        return _regenerator2.default.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                if (!services) {
                  services = [];
                }

                _context10.next = 3;
                return exec('logs', ['-f', '--tail=250'].concat((0, _toConsumableArray3.default)(services)), { pipeLog: true, maxBuffer: Infinity });

              case 3:
              case 'end':
                return _context10.stop();
            }
          }
        }, _callee10, _this10);
      }))();
    },
    port: function port(service, privatePort, index) {
      var _this11 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11() {
        var container, portConfig;
        return _regenerator2.default.wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                if (index == null) index = 1;

                _context11.next = 3;
                return _this11.ps(service);

              case 3:
                container = _context11.sent.pop();

                if (container) {
                  _context11.next = 6;
                  break;
                }

                return _context11.abrupt('return', null);

              case 6:
                portConfig = container.raw.NetworkSettings.Ports[privatePort + '/tcp'];

                if (!(!Array.isArray(portConfig) || portConfig.length === 0)) {
                  _context11.next = 9;
                  break;
                }

                return _context11.abrupt('return', null);

              case 9:
                return _context11.abrupt('return', Number(portConfig[0].HostPort));

              case 10:
              case 'end':
                return _context11.stop();
            }
          }
        }, _callee11, _this11);
      }))();
    },
    writeConfig: function writeConfig(config) {
      var _this12 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12() {
        var yamlOut;
        return _regenerator2.default.wrap(function _callee12$(_context12) {
          while (1) {
            switch (_context12.prev = _context12.next) {
              case 0:
                yamlOut = _jsYaml2.default.safeDump(config, { skipInvalid: true });
                _context12.next = 3;
                return _fs2.default.writeFileAsync(getCompiledDockerComposePath(), yamlOut);

              case 3:

                debug('Wrote docker-compose.tmp.yml', yamlOut);

              case 4:
              case 'end':
                return _context12.stop();
            }
          }
        }, _callee12, _this12);
      }))();
    },
    getConfig: function getConfig() {
      var _this13 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee13() {
        var output, config;
        return _regenerator2.default.wrap(function _callee13$(_context13) {
          while (1) {
            switch (_context13.prev = _context13.next) {
              case 0:
                _context13.next = 2;
                return exec('config', [], { useOriginalDockerComposeFiles: true, noLog: true });

              case 2:
                output = _context13.sent;
                config = _jsYaml2.default.safeLoad(output);
                return _context13.abrupt('return', config);

              case 5:
              case 'end':
                return _context13.stop();
            }
          }
        }, _callee13, _this13);
      }))();
    },
    getLaunchedServiceNames: function getLaunchedServiceNames() {
      var _this14 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee14() {
        var projectName, ps, names;
        return _regenerator2.default.wrap(function _callee14$(_context14) {
          while (1) {
            switch (_context14.prev = _context14.next) {
              case 0:
                projectName = navy.normalisedName;
                _context14.next = 3;
                return _dockerClient2.default.listContainers({
                  all: true,
                  filters: {
                    label: ['com.docker.compose.project=' + projectName]
                  }
                });

              case 3:
                ps = _context14.sent;
                names = ps.map(function (container) {
                  return container.Labels['com.docker.compose.service'];
                });
                return _context14.abrupt('return', names);

              case 6:
              case 'end':
                return _context14.stop();
            }
          }
        }, _callee14, _this14);
      }))();
    },
    getAvailableServiceNames: function getAvailableServiceNames() {
      var _this15 = this;

      return (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee15() {
        var config;
        return _regenerator2.default.wrap(function _callee15$(_context15) {
          while (1) {
            switch (_context15.prev = _context15.next) {
              case 0:
                _context15.next = 2;
                return driver.getConfig();

              case 2:
                config = _context15.sent;
                return _context15.abrupt('return', (0, _keys2.default)(config.services));

              case 4:
              case 'end':
                return _context15.stop();
            }
          }
        }, _callee15, _this15);
      }))();
    }
  };

  return driver;
}

function toLookupTable(keys) {
  var lookupTable = {};
  keys.forEach(function (key) {
    return lookupTable[key] = true;
  });
  return lookupTable;
}
module.exports = exports.default;