'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _chai = require('chai');

var _serviceHost = require('../service-host');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-env mocha */

describe('service-host', function () {

  describe('getNIPSubdomain', function () {

    it('should return an nip.io domain with the correct IP address', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.t0 = _chai.expect;
              _context.next = 3;
              return (0, _serviceHost.getNIPSubdomain)('192.168.1.10');

            case 3:
              _context.t1 = _context.sent;
              (0, _context.t0)(_context.t1).to.equal('192.168.1.10.nip.io');

            case 5:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    })));

    it('should fallback to localhost when a hostname is passed instead of an IP address', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              process.env.NAVY_EXTERNAL_IP = 'somehostname';
              _context2.t0 = _chai.expect;
              _context2.next = 4;
              return (0, _serviceHost.getNIPSubdomain)('somehostname');

            case 4:
              _context2.t1 = _context2.sent;
              (0, _context2.t0)(_context2.t1).to.equal('127.0.0.1.nip.io');

            case 6:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, this);
    })));
  });

  describe('createHostForService', function () {

    it('should return the correct host for a service', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.t0 = _chai.expect;
              _context3.next = 3;
              return (0, _serviceHost.createHostForService)('someservice', 'mynavy', '127.0.0.1');

            case 3:
              _context3.t1 = _context3.sent;
              (0, _context3.t0)(_context3.t1).to.equal('someservice.mynavy.127.0.0.1.nip.io');
              _context3.t2 = _chai.expect;
              _context3.next = 8;
              return (0, _serviceHost.createHostForService)('someservice', 'mynavy', '192.168.1.10');

            case 8:
              _context3.t3 = _context3.sent;
              (0, _context3.t2)(_context3.t3).to.equal('someservice.mynavy.192.168.1.10.nip.io');

            case 10:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, this);
    })));
  });

  describe('createUrlForService', function () {

    it('should return the correct url for a service', (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.t0 = _chai.expect;
              _context4.next = 3;
              return (0, _serviceHost.createUrlForService)('someservice', 'mynavy', '192.168.99.100');

            case 3:
              _context4.t1 = _context4.sent;
              (0, _context4.t0)(_context4.t1).to.equal('http://someservice.mynavy.192.168.99.100.nip.io');

            case 5:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, this);
    })));
  });

  describe('getUrlFromService', function () {

    it('should extract the host from service ENV config', function () {
      var service = {
        raw: {
          Config: {
            Env: ['VIRTUAL_HOST=myservice.coolnavy.127.0.0.1.nip.io']
          }
        }
      };

      (0, _chai.expect)((0, _serviceHost.getUrlFromService)(service)).to.equal('http://myservice.coolnavy.127.0.0.1.nip.io');
    });

    it('should return null when the service object is not valid', function () {
      var service = {
        raw: {
          Config: {}
        }
      };

      (0, _chai.expect)((0, _serviceHost.getUrlFromService)(service)).to.equal(null);
      (0, _chai.expect)((0, _serviceHost.getUrlFromService)(undefined)).to.equal(null);
    });
  });
});